@@ -25,7 +25,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_write_disk.c,v 1.12 2007/05/29 01:00:19 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_write_disk.c,v 1.21 2008/01/18 06:16:08 kientzle Exp $");
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -171,6 +171,8 @@ struct archive_write_disk {
 	int			 fd;
 	/* Current offset for writing data to the file. */
 	off_t			 offset;
+	/* Maximum size of file. */
+	off_t			 filesize;
 	/* Dir we were in before this restore; only for deep paths. */
 	int			 restore_pwd;
 	/* Mode we should use for this entry; affected by _PERM and umask. */
@@ -302,6 +304,7 @@ _archive_write_header(struct archive *_a, struct archive_entry *entry)
 	a->offset = 0;
 	a->uid = a->user_uid;
 	a->mode = archive_entry_mode(a->entry);
+	a->filesize = archive_entry_size(a->entry);
 	archive_strcpy(&(a->_name_data), archive_entry_pathname(a->entry));
 	a->name = a->_name_data.s;
 	archive_clear_error(&a->archive);
@@ -421,6 +424,14 @@ _archive_write_header(struct archive *_a, struct archive_entry *entry)
 	/* We've created the object and are ready to pour data into it. */
 	if (ret == ARCHIVE_OK)
 		a->archive.state = ARCHIVE_STATE_DATA;
+	/*
+	 * If it's not open, tell our client not to try writing.
+	 * In particular, dirs, links, etc, don't get written to.
+	 */
+	if (a->fd < 0) {
+		archive_entry_set_size(entry, 0);
+		a->filesize = 0;
+	}
 done:
 	/* Restore the user's umask before returning. */
 	umask(a->user_umask);
@@ -445,11 +456,14 @@ _archive_write_data_block(struct archive *_a,
 {
 	struct archive_write_disk *a = (struct archive_write_disk *)_a;
 	ssize_t bytes_written = 0;
+	int r = ARCHIVE_OK;
 
 	__archive_check_magic(&a->archive, ARCHIVE_WRITE_DISK_MAGIC,
 	    ARCHIVE_STATE_DATA, "archive_write_disk_block");
-	if (a->fd < 0)
-		return (ARCHIVE_OK);
+	if (a->fd < 0) {
+		archive_set_error(&a->archive, 0, "File not open");
+		return (ARCHIVE_WARN);
+	}
 	archive_clear_error(&a->archive);
 
 	/* Seek if necessary to the specified offset. */
@@ -462,7 +476,13 @@ _archive_write_data_block(struct archive *_a,
 	}
 
 	/* Write the data. */
-	while (size > 0) {
+	while (size > 0 && a->offset < a->filesize) {
+		if ((off_t)(a->offset + size) > a->filesize) {
+			size = (size_t)(a->filesize - a->offset);
+			archive_set_error(&a->archive, errno,
+			    "Write request too large");
+			r = ARCHIVE_WARN;
+		}
 		bytes_written = write(a->fd, buff, size);
 		if (bytes_written < 0) {
 			archive_set_error(&a->archive, errno, "Write failed");
@@ -471,19 +491,26 @@ _archive_write_data_block(struct archive *_a,
 		size -= bytes_written;
 		a->offset += bytes_written;
 	}
-	return (ARCHIVE_OK);
+	return (r);
 }
 
 static ssize_t
 _archive_write_data(struct archive *_a, const void *buff, size_t size)
 {
 	struct archive_write_disk *a = (struct archive_write_disk *)_a;
+	off_t offset;
+	int r;
+
 	__archive_check_magic(&a->archive, ARCHIVE_WRITE_DISK_MAGIC,
 	    ARCHIVE_STATE_DATA, "archive_write_data");
 	if (a->fd < 0)
 		return (ARCHIVE_OK);
 
-	return (_archive_write_data_block(_a, buff, size, a->offset));
+	offset = a->offset;
+	r = _archive_write_data_block(_a, buff, size, a->offset);
+	if (r < ARCHIVE_OK)
+		return (r);
+	return (a->offset - offset);
 }
 
 static int
@@ -610,7 +637,10 @@ archive_write_disk_new(void)
 	a->lookup_uid = trivial_lookup_uid;
 	a->lookup_gid = trivial_lookup_gid;
 	a->user_uid = geteuid();
-	archive_string_ensure(&a->path_safe, 64);
+	if (archive_string_ensure(&a->path_safe, 512) == NULL) {
+		free(a);
+		return (NULL);
+	}
 	return (&a->archive);
 }
 
@@ -813,8 +843,20 @@ create_filesystem_object(struct archive_write_disk *a)
 	/* We identify hard/symlinks according to the link names. */
 	/* Since link(2) and symlink(2) don't handle modes, we're done here. */
 	linkname = archive_entry_hardlink(a->entry);
-	if (linkname != NULL)
-		return link(linkname, a->name) ? errno : 0;
+	if (linkname != NULL) {
+		r = link(linkname, a->name) ? errno : 0;
+		/*
+		 * New cpio and pax formats allow hardlink entries
+		 * to carry data, so we may have to open the file
+		 * for hardlink entries.
+		 */
+		if (r == 0 && a->filesize > 0) {
+			a->fd = open(a->name, O_WRONLY | O_TRUNC);
+			if (a->fd < 0)
+				r = errno;
+		}
+		return (r);
+	}
 	linkname = archive_entry_symlink(a->entry);
 	if (linkname != NULL)
 		return symlink(linkname, a->name) ? errno : 0;
@@ -1538,15 +1580,28 @@ set_mode(struct archive_write_disk *a, int mode)
 		}
 		if (a->pst->st_gid != a->gid) {
 			mode &= ~ S_ISGID;
-			archive_set_error(&a->archive, -1, "Can't restore SGID bit");
-			r = ARCHIVE_WARN;
+			if (a->flags & ARCHIVE_EXTRACT_OWNER) {
+				/*
+				 * This is only an error if you
+				 * requested owner restore.  If you
+				 * didn't, we'll try to restore
+				 * sgid/suid, but won't consider it a
+				 * problem if we can't.
+				 */
+				archive_set_error(&a->archive, -1,
+				    "Can't restore SGID bit");
+				r = ARCHIVE_WARN;
+			}
 		}
 		/* While we're here, double-check the UID. */
 		if (a->pst->st_uid != a->uid
 		    && (a->todo & TODO_SUID)) {
 			mode &= ~ S_ISUID;
-			archive_set_error(&a->archive, -1, "Can't restore SUID bit");
-			r = ARCHIVE_WARN;
+			if (a->flags & ARCHIVE_EXTRACT_OWNER) {
+				archive_set_error(&a->archive, -1,
+				    "Can't restore SUID bit");
+				r = ARCHIVE_WARN;
+			}
 		}
 		a->todo &= ~TODO_SGID_CHECK;
 		a->todo &= ~TODO_SUID_CHECK;
@@ -1558,8 +1613,11 @@ set_mode(struct archive_write_disk *a, int mode)
 		 */
 		if (a->user_uid != a->uid) {
 			mode &= ~ S_ISUID;
-			archive_set_error(&a->archive, -1, "Can't make file SUID");
-			r = ARCHIVE_WARN;
+			if (a->flags & ARCHIVE_EXTRACT_OWNER) {
+				archive_set_error(&a->archive, -1,
+				    "Can't make file SUID");
+				r = ARCHIVE_WARN;
+			}
 		}
 		a->todo &= ~TODO_SUID_CHECK;
 	}
